"""
☆☆☆ 指定されたフォルダ内の全PDFファイルを指定されたパスワードで暗号化するツール ☆☆☆
1. pipコマンドをアップデートする。
   C:\> python -m pip install --upgrade pip
2. パッケージpikepdfをインストールする。
   C:\> pip install pikepdf
3. 実行する。
   C:\> python encryptPDF.py -f Folder -p Password [-nb]
        -f Folder: 暗号化するPDFファイルが格納されているフォルダ
        -p Password: 暗号化に使用するパスワード
        -nb: 暗号化時に元のPDFファイルをバックアップしないことを指定するフラグ
"""

import sys
import os
import glob
import shutil
import itertools
import argparse
import pikepdf

def encrypt_pdf(folder: str, password: str, noBackup: bool) -> None:
    """指定されたフォルダ内の全PDFファイルを指定されたパスワードで暗号化する。

    Args:
        folder (str): 暗号化するPDFファイルが格納されているフォルダ
        password (str): 暗号化に使用するパスワード
        noBackup (bool): 暗号化時に元のPDFファイルをバックアップしないことを指定するフラグ
    """
    # PDFファイルの一覧を取得する。
    pdfs = [p.replace(os.sep, '/') for p in glob.glob(folder + '/*.pdf')]

    # それぞれのPDFファイルについて・・・。
    for pdf in pdfs:
        print('**** ' + pdf)
        print('---> 暗号化を開始します。')

        # PDFファイルを開く。
        with pikepdf.open(pdf) as org:
            # PDFファイルが既に暗号化されていたら処理をスキップする。
            if org.is_encrypted:
                print('---> 既に暗号化されています。')
                continue

        # PDFファイルをバックアップ(リネーム)する。
        bak = search_alternate_filepath(pdf.removesuffix('.pdf') + '.bak')
        shutil.move(pdf, bak)

        # バックアップ(リネーム)したPDFファイルを暗号化し元のPDFファイルのパスで保存する。
        with pikepdf.open(bak) as org:
            org.save(pdf, encryption=pikepdf.Encryption(user=password))

        # noBackupが指定されている場合バックアップ(リネーム)したPDFファイルを削除する。
        if noBackup:
            os.remove(bak)

        print('---> 暗号化が終了しました。')

def search_alternate_filepath(filepath: str) -> str:
    """バックアップ先などに使う代替のファイルパスを検索する。

    Args:
        filepath (str): オリジナルのファイルパス

    Returns:
        str: 発見した代替のファイルパス
    """
    # ファイルパスをベース名(フルパス)と拡張子に分割する。
    base, ext = os.path.splitext(filepath)

    # iが1から始まる無限ループを回す。
    for i in itertools.count(1):
        # 変数filepathが示すファイルが存在していたら・・・。
        if os.path.exists(filepath):
            # 次の候補のファイルパスを作成する。
            filepath = base + '({})'.format(i) + ext
        else:
            # ファイルが存在しないならここで終了。
            return filepath

if __name__ == '__main__':
    """コマンドライン引数を解析してメインルーチンを呼び出す。
    """
    # コマンドライン引数を解析する。
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', '--folder', type=str, required=True, metavar='Folder')
    parser.add_argument('-p', '--password', type=str, required=True, metavar='Password')
    parser.add_argument('-nb', '--nobackup', action='store_true')
    args = parser.parse_args()

    # 指定されたフォルダが存在しなければプログラムを終了する。
    if not os.path.isdir(args.folder):
        print('The specified folder does not exist.')
        parser.print_usage()
        sys.exit(255)

    # メインルーチンを呼び出す。
    encrypt_pdf(args.folder, args.password, args.nobackup)
